@@ -11,60 +11,88 @@ Inheritance diagram:
 # Distributed under the terms of the Modified BSD License.
 
 import abc
+import inspect
+import json
 import sys
 import traceback
 import warnings
-from io import StringIO
 
 from decorator import decorator
 
 from traitlets.config.configurable import Configurable
-from .getipython import get_ipython
-from ..utils.sentinel import Sentinel
-from ..utils.dir2 import get_real_method
-from ..lib import pretty
+from IPython.core.getipython import get_ipython
+from IPython.utils.sentinel import Sentinel
+from IPython.lib import pretty
 from traitlets import (
     Bool, Dict, Integer, Unicode, CUnicode, ObjectName, List,
     ForwardDeclaredInstance,
-    default, observe,
 )
+from IPython.utils.py3compat import (
+    with_metaclass, string_types, unicode_type,
+)
+
+
+#-----------------------------------------------------------------------------
+# The main DisplayFormatter class
+#-----------------------------------------------------------------------------
+
+
+def _safe_get_formatter_method(obj, name):
+    """Safely get a formatter method
+    
+    - Classes cannot have formatter methods, only instance
+    - protect against proxy objects that claim to have everything
+    """
+    if inspect.isclass(obj):
+        # repr methods only make sense on instances, not classes
+        return None
+    method = pretty._safe_getattr(obj, name, None)
+    if callable(method):
+        # obj claims to have repr method...
+        if callable(pretty._safe_getattr(obj, '_ipython_canary_method_should_not_exist_', None)):
+            # ...but don't trust proxy objects that claim to have everything
+            return None
+        return method
 
 
 class DisplayFormatter(Configurable):
 
-    active_types = List(Unicode(),
+    # When set to true only the default plain text formatter will be used.
+    plain_text_only = Bool(False, config=True)
+    def _plain_text_only_changed(self, name, old, new):
+        warnings.warn("""DisplayFormatter.plain_text_only is deprecated.
+        
+        Use DisplayFormatter.active_types = ['text/plain']
+        for the same effect.
+        """, DeprecationWarning)
+        if new:
+            self.active_types = ['text/plain']
+        else:
+            self.active_types = self.format_types
+    
+    active_types = List(Unicode(), config=True,
         help="""List of currently active mime-types to display.
         You can use this to set a white-list for formats to display.
         
         Most users will not need to change this value.
-        """).tag(config=True)
-
-    @default('active_types')
+        """)
     def _active_types_default(self):
         return self.format_types
-
-    @observe('active_types')
-    def _active_types_changed(self, change):
+    
+    def _active_types_changed(self, name, old, new):
         for key, formatter in self.formatters.items():
-            if key in change['new']:
+            if key in new:
                 formatter.enabled = True
             else:
                 formatter.enabled = False
-
+    
     ipython_display_formatter = ForwardDeclaredInstance('FormatterABC')
-    @default('ipython_display_formatter')
-    def _default_formatter(self):
+    def _ipython_display_formatter_default(self):
         return IPythonDisplayFormatter(parent=self)
-
-    mimebundle_formatter = ForwardDeclaredInstance('FormatterABC')
-    @default('mimebundle_formatter')
-    def _default_mime_formatter(self):
-        return MimeBundleFormatter(parent=self)
-
+    
     # A dict of formatter whose keys are format types (MIME types) and whose
     # values are subclasses of BaseFormatter.
     formatters = Dict()
-    @default('formatters')
     def _formatters_default(self):
         """Activate the default formatters."""
         formatter_classes = [
@@ -90,7 +118,7 @@ class DisplayFormatter(Configurable):
 
         By default all format types will be computed.
 
-        The following MIME types are usually implemented:
+        The following MIME types are currently implemented:
 
         * text/plain
         * text/html
@@ -107,35 +135,27 @@ class DisplayFormatter(Configurable):
         ----------
         obj : object
             The Python object whose format data will be computed.
-        include : list, tuple or set; optional
+        include : list or tuple, optional
             A list of format type strings (MIME types) to include in the
             format data dict. If this is set *only* the format types included
             in this list will be computed.
-        exclude : list, tuple or set; optional
+        exclude : list or tuple, optional
             A list of format type string (MIME types) to exclude in the format
             data dict. If this is set all format types will be computed,
             except for those included in this argument.
-            Mimetypes present in exclude will take precedence over the ones in include
 
         Returns
         -------
         (format_dict, metadata_dict) : tuple of two dicts
+        
             format_dict is a dictionary of key/value pairs, one of each format that was
             generated for the object. The keys are the format types, which
             will usually be MIME type strings and the values and JSON'able
             data structure containing the raw data for the representation in
             that format.
-
+            
             metadata_dict is a dictionary of metadata about each mime-type output.
             Its keys will be a strict subset of the keys in format_dict.
-
-        Notes
-        -----
-            If an object implement `_repr_mimebundle_` as well as various
-            `_repr_*_`, the data returned by `_repr_mimebundle_` will take
-            precedence and the corresponding `_repr_*_` for this mimetype will
-            not be called.
-
         """
         format_dict = {}
         md_dict = {}
@@ -143,30 +163,8 @@ class DisplayFormatter(Configurable):
         if self.ipython_display_formatter(obj):
             # object handled itself, don't proceed
             return {}, {}
-
-        format_dict, md_dict = self.mimebundle_formatter(obj, include=include, exclude=exclude)
-
-        if format_dict or md_dict:
-            if include:
-                format_dict = {k:v for k,v in format_dict.items() if k in include}
-                md_dict = {k:v for k,v in md_dict.items() if k in include}
-            if exclude:
-                format_dict = {k:v for k,v in format_dict.items() if k not in exclude}
-                md_dict = {k:v for k,v in md_dict.items() if k not in exclude}
-
+        
         for format_type, formatter in self.formatters.items():
-            if format_type in format_dict:
-                # already got it from mimebundle, maybe don't render again.
-                # exception: manually registered per-mime renderer
-                # check priority:
-                # 1. user-registered per-mime formatter
-                # 2. mime-bundle (user-registered or repr method)
-                # 3. default per-mime formatter (e.g. repr method)
-                try:
-                    formatter.lookup(obj)
-                except KeyError:
-                    # no special formatter, use mime-bundle-provided value
-                    continue
             if include and format_type not in include:
                 continue
             if exclude and format_type in exclude:
@@ -187,6 +185,7 @@ class DisplayFormatter(Configurable):
                 format_dict[format_type] = data
             if md is not None:
                 md_dict[format_type] = md
+            
         return format_dict, md_dict
 
     @property
@@ -221,7 +220,7 @@ def catch_format_error(method, self, *args, **kwargs):
         r = method(self, *args, **kwargs)
     except NotImplementedError:
         # don't warn on NotImplementedErrors
-        return self._check_return(None, args[0])
+        return None
     except Exception:
         exc_info = sys.exc_info()
         ip = get_ipython()
@@ -229,11 +228,11 @@ def catch_format_error(method, self, *args, **kwargs):
             ip.showtraceback(exc_info)
         else:
             traceback.print_exception(*exc_info)
-        return self._check_return(None, args[0])
+        return None
     return self._check_return(r, args[0])
 
 
-class FormatterABC(metaclass=abc.ABCMeta):
+class FormatterABC(with_metaclass(abc.ABCMeta, object)):
     """ Abstract base class for Formatters.
 
     A formatter is a callable class that is responsible for computing the
@@ -260,7 +259,7 @@ class FormatterABC(metaclass=abc.ABCMeta):
 
 def _mod_name_key(typ):
     """Return a (__module__, __name__) tuple for a type.
-
+    
     Used as key in Formatter.deferred_printers.
     """
     module = getattr(typ, '__module__', None)
@@ -307,23 +306,23 @@ class BaseFormatter(Configurable):
     """
 
     format_type = Unicode('text/plain')
-    _return_type = str
+    _return_type = string_types
 
-    enabled = Bool(True).tag(config=True)
+    enabled = Bool(True, config=True)
 
     print_method = ObjectName('__repr__')
 
     # The singleton printers.
     # Maps the IDs of the builtin singleton objects to the format functions.
-    singleton_printers = Dict().tag(config=True)
+    singleton_printers = Dict(config=True)
 
     # The type-specific printers.
     # Map type objects to the format functions.
-    type_printers = Dict().tag(config=True)
+    type_printers = Dict(config=True)
 
     # The deferred-import type-specific printers.
     # Map (modulename, classname) pairs to the format functions.
-    deferred_printers = Dict().tag(config=True)
+    deferred_printers = Dict(config=True)
     
     @catch_format_error
     def __call__(self, obj):
@@ -337,7 +336,7 @@ class BaseFormatter(Configurable):
             else:
                 return printer(obj)
             # Finally look for special method names
-            method = get_real_method(obj, self.print_method)
+            method = _safe_get_formatter_method(obj, self.print_method)
             if method is not None:
                 return method()
             return None
@@ -355,7 +354,7 @@ class BaseFormatter(Configurable):
     
     def _check_return(self, r, obj):
         """Check that a return value is appropriate
-
+        
         Return the value if so, None otherwise, warning if invalid.
         """
         if r is None or isinstance(r, self._return_type) or \
@@ -370,10 +369,10 @@ class BaseFormatter(Configurable):
     
     def lookup(self, obj):
         """Look up the formatter for a given instance.
-
+        
         Parameters
         ----------
-        obj : object instance
+        obj  : object instance
 
         Returns
         -------
@@ -396,7 +395,7 @@ class BaseFormatter(Configurable):
 
         Parameters
         ----------
-        typ : type or '__module__.__name__' string for a type
+        typ  : type or '__module__.__name__' string for a type
 
         Returns
         -------
@@ -407,7 +406,7 @@ class BaseFormatter(Configurable):
         ------
         KeyError if the type has not been registered.
         """
-        if isinstance(typ, str):
+        if isinstance(typ, string_types):
             typ_key = tuple(typ.rsplit('.',1))
             if typ_key not in self.deferred_printers:
                 # We may have it cached in the type map. We will have to
@@ -427,22 +426,21 @@ class BaseFormatter(Configurable):
 
     def for_type(self, typ, func=None):
         """Add a format function for a given type.
-
+        
         Parameters
-        ----------
+        -----------
         typ : type or '__module__.__name__' string for a type
             The class of the object that will be formatted using `func`.
-
         func : callable
             A callable for computing the format data.
             `func` will be called with the object to be formatted,
             and will return the raw data in this formatter's format.
             Subclasses may use a different call signature for the
             `func` argument.
-
+            
             If `func` is None or not specified, there will be no change,
             only returning the current value.
-
+        
         Returns
         -------
         oldfunc : callable
@@ -451,7 +449,7 @@ class BaseFormatter(Configurable):
             this will be the previous value (to enable restoring later).
         """
         # if string given, interpret as 'pkg.module.class_name'
-        if isinstance(typ, str):
+        if isinstance(typ, string_types):
             type_module, type_name = typ.rsplit('.', 1)
             return self.for_type_by_name(type_module, type_name, func)
         
@@ -474,20 +472,18 @@ class BaseFormatter(Configurable):
         type_module : str
             The full dotted name of the module the type is defined in, like
             ``numpy``.
-
         type_name : str
             The name of the type (the class name), like ``dtype``
-
         func : callable
             A callable for computing the format data.
             `func` will be called with the object to be formatted,
             and will return the raw data in this formatter's format.
             Subclasses may use a different call signature for the
             `func` argument.
-
+            
             If `func` is None or unspecified, there will be no change,
             only returning the current value.
-
+        
         Returns
         -------
         oldfunc : callable
@@ -525,7 +521,7 @@ class BaseFormatter(Configurable):
         KeyError if the type is not registered and default is not specified.
         """
         
-        if isinstance(typ, str):
+        if isinstance(typ, string_types):
             typ_key = tuple(typ.rsplit('.',1))
             if typ_key not in self.deferred_printers:
                 # We may have it cached in the type map. We will have to
@@ -593,37 +589,36 @@ class PlainTextFormatter(BaseFormatter):
 
     # This subclass ignores this attribute as it always need to return
     # something.
-    enabled = Bool(True).tag(config=False)
+    enabled = Bool(True, config=False)
     
-    max_seq_length = Integer(pretty.MAX_SEQ_LENGTH,
+    max_seq_length = Integer(pretty.MAX_SEQ_LENGTH, config=True,
         help="""Truncate large collections (lists, dicts, tuples, sets) to this size.
         
         Set to 0 to disable truncation.
         """
-    ).tag(config=True)
+    )
     
     # Look for a _repr_pretty_ methods to use for pretty printing.
     print_method = ObjectName('_repr_pretty_')
 
     # Whether to pretty-print or not.
-    pprint = Bool(True).tag(config=True)
+    pprint = Bool(True, config=True)
 
     # Whether to be verbose or not.
-    verbose = Bool(False).tag(config=True)
+    verbose = Bool(False, config=True)
 
     # The maximum width.
-    max_width = Integer(79).tag(config=True)
+    max_width = Integer(79, config=True)
 
     # The newline character.
-    newline = Unicode('\n').tag(config=True)
+    newline = Unicode('\n', config=True)
 
     # format-string for pprinting floats
     float_format = Unicode('%r')
     # setter for float precision, either int or direct format-string
-    float_precision = CUnicode('').tag(config=True)
+    float_precision = CUnicode('', config=True)
 
-    @observe('float_precision')
-    def _float_precision_changed(self, change):
+    def _float_precision_changed(self, name, old, new):
         """float_precision changed, set float_format accordingly.
 
         float_precision can be set by int or str.
@@ -636,23 +631,23 @@ class PlainTextFormatter(BaseFormatter):
 
         This parameter can be set via the '%precision' magic.
         """
-        new = change['new']
+
         if '%' in new:
             # got explicit format string
             fmt = new
             try:
                 fmt%3.14159
-            except Exception as e:
-                raise ValueError("Precision must be int or format string, not %r"%new) from e
+            except Exception:
+                raise ValueError("Precision must be int or format string, not %r"%new)
         elif new:
             # otherwise, should be an int
             try:
                 i = int(new)
                 assert i >= 0
-            except ValueError as e:
-                raise ValueError("Precision must be int or format string, not %r"%new) from e
-            except AssertionError as e:
-                raise ValueError("int precision must be non-negative, not %r"%i) from e
+            except ValueError:
+                raise ValueError("Precision must be int or format string, not %r"%new)
+            except AssertionError:
+                raise ValueError("int precision must be non-negative, not %r"%i)
 
             fmt = '%%.%if'%i
             if 'numpy' in sys.modules:
@@ -669,22 +664,14 @@ class PlainTextFormatter(BaseFormatter):
         self.float_format = fmt
 
     # Use the default pretty printers from IPython.lib.pretty.
-    @default('singleton_printers')
     def _singleton_printers_default(self):
         return pretty._singleton_pprinters.copy()
 
-    @default('type_printers')
     def _type_printers_default(self):
         d = pretty._type_pprinters.copy()
         d[float] = lambda obj,p,cycle: p.text(self.float_format%obj)
-        # if NumPy is used, set precision for its float64 type
-        if "numpy" in sys.modules:
-            import numpy
-
-            d[numpy.float64] = lambda obj, p, cycle: p.text(self.float_format % obj)
         return d
 
-    @default('deferred_printers')
     def _deferred_printers_default(self):
         return pretty._deferred_type_pprinters.copy()
 
@@ -696,7 +683,11 @@ class PlainTextFormatter(BaseFormatter):
         if not self.pprint:
             return repr(obj)
         else:
-            stream = StringIO()
+            # handle str and unicode on Python 2
+            # io.StringIO only accepts unicode,
+            # cStringIO doesn't handle unicode on py2,
+            # StringIO allows str, unicode but only ascii str
+            stream = pretty.CUnicodeIO()
             printer = pretty.RepresentationPrinter(stream, self.verbose,
                 self.max_width, self.newline,
                 max_seq_length=self.max_seq_length,
@@ -771,7 +762,7 @@ class PNGFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_png_')
     
-    _return_type = (bytes, str)
+    _return_type = (bytes, unicode_type)
 
 
 class JPEGFormatter(BaseFormatter):
@@ -789,7 +780,7 @@ class JPEGFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_jpeg_')
 
-    _return_type = (bytes, str)
+    _return_type = (bytes, unicode_type)
 
 
 class LatexFormatter(BaseFormatter):
@@ -827,7 +818,7 @@ class JSONFormatter(BaseFormatter):
     
     def _check_return(self, r, obj):
         """Check that a return value is appropriate
-
+        
         Return the value if so, None otherwise, warning if invalid.
         """
         if r is None:
@@ -836,11 +827,13 @@ class JSONFormatter(BaseFormatter):
         if isinstance(r, tuple):
             # unpack data, metadata tuple for type checking on first element
             r, md = r
-
-        assert not isinstance(
-            r, str
-        ), "JSON-as-string has been deprecated since IPython < 3"
-
+        
+        # handle deprecated JSON-as-string form from IPython < 3
+        if isinstance(r, string_types):
+            warnings.warn("JSON expects JSONable list/dict containers, not JSON strings",
+            FormatterWarning)
+            r = json.loads(r)
+        
         if md is not None:
             # put the tuple back together
             r = (r, md)
@@ -878,10 +871,10 @@ class PDFFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_pdf_')
 
-    _return_type = (bytes, str)
+    _return_type = (bytes, unicode_type)
 
 class IPythonDisplayFormatter(BaseFormatter):
-    """An escape-hatch Formatter for objects that know how to display themselves.
+    """A Formatter for objects that know how to display themselves.
     
     To define the callables that compute the representation of your
     objects, define a :meth:`_ipython_display_` method or use the :meth:`for_type`
@@ -891,16 +884,10 @@ class IPythonDisplayFormatter(BaseFormatter):
     
     This display formatter has highest priority.
     If it fires, no other display formatter will be called.
-
-    Prior to IPython 6.1, `_ipython_display_` was the only way to display custom mime-types
-    without registering a new Formatter.
-    
-    IPython 6.1 introduces `_repr_mimebundle_` for displaying custom mime-types,
-    so `_ipython_display_` should only be used for objects that require unusual
-    display patterns, such as multiple display calls.
     """
     print_method = ObjectName('_ipython_display_')
     _return_type = (type(None), bool)
+    
 
     @catch_format_error
     def __call__(self, obj):
@@ -915,66 +902,12 @@ class IPythonDisplayFormatter(BaseFormatter):
                 printer(obj)
                 return True
             # Finally look for special method names
-            method = get_real_method(obj, self.print_method)
+            method = _safe_get_formatter_method(obj, self.print_method)
             if method is not None:
                 method()
                 return True
 
 
-class MimeBundleFormatter(BaseFormatter):
-    """A Formatter for arbitrary mime-types.
-
-    Unlike other `_repr_<mimetype>_` methods,
-    `_repr_mimebundle_` should return mime-bundle data,
-    either the mime-keyed `data` dictionary or the tuple `(data, metadata)`.
-    Any mime-type is valid.
-
-    To define the callables that compute the mime-bundle representation of your
-    objects, define a :meth:`_repr_mimebundle_` method or use the :meth:`for_type`
-    or :meth:`for_type_by_name` methods to register functions that handle
-    this.
-
-    .. versionadded:: 6.1
-    """
-    print_method = ObjectName('_repr_mimebundle_')
-    _return_type = dict
-    
-    def _check_return(self, r, obj):
-        r = super(MimeBundleFormatter, self)._check_return(r, obj)
-        # always return (data, metadata):
-        if r is None:
-            return {}, {}
-        if not isinstance(r, tuple):
-            return r, {}
-        return r
-
-    @catch_format_error
-    def __call__(self, obj, include=None, exclude=None):
-        """Compute the format for an object.
-
-        Identical to parent's method but we pass extra parameters to the method.
-
-        Unlike other _repr_*_ `_repr_mimebundle_` should allow extra kwargs, in
-        particular `include` and `exclude`.
-        """
-        if self.enabled:
-            # lookup registered printer
-            try:
-                printer = self.lookup(obj)
-            except KeyError:
-                pass
-            else:
-                return printer(obj)
-            # Finally look for special method names
-            method = get_real_method(obj, self.print_method)
-
-            if method is not None:
-                return method(include=include, exclude=exclude)
-            return None
-        else:
-            return None
-
-
 FormatterABC.register(BaseFormatter)
 FormatterABC.register(PlainTextFormatter)
 FormatterABC.register(HTMLFormatter)
@@ -987,7 +920,6 @@ FormatterABC.register(LatexFormatter)
 FormatterABC.register(JSONFormatter)
 FormatterABC.register(JavascriptFormatter)
 FormatterABC.register(IPythonDisplayFormatter)
-FormatterABC.register(MimeBundleFormatter)
 
 
 def format_display_data(obj, include=None, exclude=None):
@@ -995,6 +927,19 @@ def format_display_data(obj, include=None, exclude=None):
 
     By default all format types will be computed.
 
+    The following MIME types are currently implemented:
+
+    * text/plain
+    * text/html
+    * text/markdown
+    * text/latex
+    * application/json
+    * application/javascript
+    * application/pdf
+    * image/png
+    * image/jpeg
+    * image/svg+xml
+
     Parameters
     ----------
     obj : object
@@ -1013,14 +958,15 @@ def format_display_data(obj, include=None, exclude=None):
         format data dict. If this is set *only* the format types included
         in this list will be computed.
     exclude : list or tuple, optional
-        A list of format type string (MIME types) to exclude in the format
+        A list of format type string (MIME types) to exclue in the format
         data dict. If this is set all format types will be computed,
         except for those included in this argument.
     """
-    from .interactiveshell import InteractiveShell
+    from IPython.core.interactiveshell import InteractiveShell
 
-    return InteractiveShell.instance().display_formatter.format(
+    InteractiveShell.instance().display_formatter.format(
         obj,
         include,
         exclude
     )
+
