@@ -1,19 +1,22 @@
 # -*- coding: utf-8 -*-
 """Tests for the inputsplitter module."""
 
+from __future__ import print_function
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
 import unittest
-import pytest
 import sys
 
-with pytest.warns(DeprecationWarning, match="inputsplitter"):
-    from IPython.core import inputsplitter as isp
+import nose.tools as nt
+
+from IPython.core import inputsplitter as isp
 from IPython.core.inputtransformer import InputTransformer
 from IPython.core.tests.test_inputtransformer import syntax, syntax_ml
 from IPython.testing import tools as tt
+from IPython.utils import py3compat
+from IPython.utils.py3compat import string_types, input
 
 #-----------------------------------------------------------------------------
 # Semi-complete examples (also used as tests)
@@ -35,7 +38,7 @@ def mini_interactive_loop(input_func):
     # input indefinitely, until some exit/quit command was issued.  Here we
     # only illustrate the basic inner loop.
     while isp.push_accepts_more():
-        indent = ' '*isp.get_indent_spaces()
+        indent = ' '*isp.indent_spaces
         prompt = '>>> ' + indent
         line = indent + input_func(prompt)
         isp.push(line)
@@ -50,6 +53,10 @@ def mini_interactive_loop(input_func):
 # Test utilities, just for local use
 #-----------------------------------------------------------------------------
 
+def assemble(block):
+    """Assemble a block into multi-line sub-blocks."""
+    return ['\n'.join(sub_block)+'\n' for sub_block in block]
+
 
 def pseudo_input(lines):
     """Return a function that acts like raw_input but feeds the input list."""
@@ -94,10 +101,10 @@ def test_remove_comments():
 
 def test_get_input_encoding():
     encoding = isp.get_input_encoding()
-    assert isinstance(encoding, str)
+    nt.assert_true(isinstance(encoding, string_types))
     # simple-minded check that at least encoding a simple string works with the
     # encoding we got.
-    assert "test".encode(encoding) == b"test"
+    nt.assert_equal(u'test'.encode(encoding), b'test')
 
 
 class NoInputEncodingTestCase(unittest.TestCase):
@@ -126,7 +133,7 @@ class InputSplitterTestCase(unittest.TestCase):
         isp.push('x=1')
         isp.reset()
         self.assertEqual(isp._buffer, [])
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         self.assertEqual(isp.source, '')
         self.assertEqual(isp.code, None)
         self.assertEqual(isp._is_complete, False)
@@ -135,7 +142,7 @@ class InputSplitterTestCase(unittest.TestCase):
         self.isp._store('1')
         self.isp._store('2')
         self.assertEqual(self.isp.source, '1\n2\n')
-        self.assertEqual(len(self.isp._buffer)>0, True)
+        self.assertTrue(len(self.isp._buffer)>0)
         self.assertEqual(self.isp.source_reset(), '1\n2\n')
         self.assertEqual(self.isp._buffer, [])
         self.assertEqual(self.isp.source, '')
@@ -143,21 +150,21 @@ class InputSplitterTestCase(unittest.TestCase):
     def test_indent(self):
         isp = self.isp # shorthand
         isp.push('x=1')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n    x=1')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('y=2\n')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_indent2(self):
         isp = self.isp
         isp.push('if 1:')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('    x=1')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         # Blank lines shouldn't change the indent level
         isp.push(' '*2)
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
 
     def test_indent3(self):
         isp = self.isp
@@ -165,111 +172,111 @@ class InputSplitterTestCase(unittest.TestCase):
         # shouldn't get confused.
         isp.push("if 1:")
         isp.push("    x = (1+\n    2)")
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
 
     def test_indent4(self):
         isp = self.isp
         # whitespace after ':' should not screw up indent level
         isp.push('if 1: \n    x=1')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('y=2\n')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\t\n    x=1')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('y=2\n')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_dedent_pass(self):
         isp = self.isp # shorthand
         # should NOT cause dedent
         isp.push('if 1:\n    passes = 5')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('if 1:\n     pass')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     pass   ')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_dedent_break(self):
         isp = self.isp # shorthand
         # should NOT cause dedent
         isp.push('while 1:\n    breaks = 5')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('while 1:\n     break')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('while 1:\n     break   ')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_dedent_continue(self):
         isp = self.isp # shorthand
         # should NOT cause dedent
         isp.push('while 1:\n    continues = 5')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('while 1:\n     continue')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('while 1:\n     continue   ')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_dedent_raise(self):
         isp = self.isp # shorthand
         # should NOT cause dedent
         isp.push('if 1:\n    raised = 4')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('if 1:\n     raise TypeError()')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     raise')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     raise      ')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_dedent_return(self):
         isp = self.isp # shorthand
         # should NOT cause dedent
         isp.push('if 1:\n    returning = 4')
-        self.assertEqual(isp.get_indent_spaces(), 4)
+        self.assertEqual(isp.indent_spaces, 4)
         isp.push('if 1:\n     return 5 + 493')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     return')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     return      ')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
         isp.push('if 1:\n     return(0)')
-        self.assertEqual(isp.get_indent_spaces(), 0)
+        self.assertEqual(isp.indent_spaces, 0)
 
     def test_push(self):
         isp = self.isp
-        self.assertEqual(isp.push('x=1'), True)
+        self.assertTrue(isp.push('x=1'))
 
     def test_push2(self):
         isp = self.isp
-        self.assertEqual(isp.push('if 1:'), False)
+        self.assertFalse(isp.push('if 1:'))
         for line in ['  x=1', '# a comment', '  y=2']:
             print(line)
-            self.assertEqual(isp.push(line), True)
+            self.assertTrue(isp.push(line))
 
     def test_push3(self):
         isp = self.isp
         isp.push('if True:')
         isp.push('  a = 1')
-        self.assertEqual(isp.push('b = [1,'), False)
+        self.assertFalse(isp.push('b = [1,'))
 
     def test_push_accepts_more(self):
         isp = self.isp
         isp.push('x=1')
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_push_accepts_more2(self):
         isp = self.isp
         isp.push('if 1:')
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
         isp.push('  x=1')
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
         isp.push('')
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_push_accepts_more3(self):
         isp = self.isp
         isp.push("x = (2+\n3)")
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_push_accepts_more4(self):
         isp = self.isp
@@ -283,11 +290,11 @@ class InputSplitterTestCase(unittest.TestCase):
         isp.push("if 1:")
         isp.push("    x = (2+")
         isp.push("    3)")
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
         isp.push("    y = 3")
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
         isp.push('')
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_push_accepts_more5(self):
         isp = self.isp
@@ -297,14 +304,14 @@ class InputSplitterTestCase(unittest.TestCase):
         isp.push('    raise')
         # We want to be able to add an else: block at this point, so it should
         # wait for a blank line.
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
 
     def test_continuation(self):
         isp = self.isp
         isp.push("import os, \\")
-        self.assertEqual(isp.push_accepts_more(), True)
+        self.assertTrue(isp.push_accepts_more())
         isp.push("sys")
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_syntax_error(self):
         isp = self.isp
@@ -312,34 +319,28 @@ class InputSplitterTestCase(unittest.TestCase):
         # Python can be sent to the kernel for evaluation with possible ipython
         # special-syntax conversion.
         isp.push('run foo')
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_unicode(self):
         self.isp.push(u"Pérez")
         self.isp.push(u'\xc3\xa9')
         self.isp.push(u"u'\xc3\xa9'")
 
-    @pytest.mark.xfail(
-        reason="Bug in python 3.9.8 – bpo 45738",
-        condition=sys.version_info in [(3, 9, 8, "final", 0), (3, 11, 0, "alpha", 2)],
-        raises=SystemError,
-        strict=True,
-    )
     def test_line_continuation(self):
         """ Test issue #2108."""
         isp = self.isp
         # A blank line after a line continuation should not accept more
         isp.push("1 \\\n\n")
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
         # Whitespace after a \ is a SyntaxError.  The only way to test that
         # here is to test that push doesn't accept more (as with
         # test_syntax_error() above).
         isp.push(r"1 \ ")
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
         # Even if the line is continuable (c.f. the regular Python
         # interpreter)
         isp.push(r"(1 \ ")
-        self.assertEqual(isp.push_accepts_more(), False)
+        self.assertFalse(isp.push_accepts_more())
 
     def test_check_complete(self):
         isp = self.isp
@@ -421,7 +422,7 @@ class IPythonInputTestCase(InputSplitterTestCase):
                 for lraw, out_t_part in line_pairs:
                     if out_t_part is not None:
                         out_t_parts.append(out_t_part)
-
+                    
                     if lraw is not None:
                         isp.push(lraw)
                         raw_parts.append(lraw)
@@ -444,14 +445,14 @@ class IPythonInputTestCase(InputSplitterTestCase):
                 out = isp.transform_cell(raw)
                 # Match ignoring trailing whitespace
                 self.assertEqual(out.rstrip(), out_t.rstrip())
-
+    
     def test_cellmagic_preempt(self):
         isp = self.isp
         for raw, name, line, cell in [
             ("%%cellm a\nIn[1]:", u'cellm', u'a', u'In[1]:'),
             ("%%cellm \nline\n>>> hi", u'cellm', u'', u'line\n>>> hi'),
             (">>> %%cellm \nline\n>>> hi", u'cellm', u'', u'line\nhi'),
-            ("%%cellm \n>>> hi", u'cellm', u'', u'>>> hi'),
+            ("%%cellm \n>>> hi", u'cellm', u'', u'hi'),
             ("%%cellm \nline1\nline2", u'cellm', u'', u'line1\nline2'),
             ("%%cellm \nline1\\\\\nline2", u'cellm', u'', u'line1\\\\\nline2'),
         ]:
@@ -466,24 +467,24 @@ class IPythonInputTestCase(InputSplitterTestCase):
         class CommentTransformer(InputTransformer):
             def __init__(self):
                 self._lines = []
-
+            
             def push(self, line):
                 self._lines.append(line + '#')
-
+            
             def reset(self):
                 text = '\n'.join(self._lines)
                 self._lines = []
                 return text
-
+        
         isp.physical_line_transforms.insert(0, CommentTransformer())
-
+        
         for raw, expected in [
             ("a=5", "a=5#"),
-            ("%ls foo", "get_ipython().run_line_magic(%r, %r)" % (u'ls', u'foo#')),
-            ("!ls foo\n%ls bar", "get_ipython().system(%r)\nget_ipython().run_line_magic(%r, %r)" % (
-                u'ls foo#', u'ls', u'bar#'
+            ("%ls foo", "get_ipython().magic(%r)" % u'ls foo#'),
+            ("!ls foo\n%ls bar", "get_ipython().system(%r)\nget_ipython().magic(%r)" % (
+                u'ls foo#', u'ls bar#'
             )),
-            ("1\n2\n3\n%ls foo\n4\n5", "1#\n2#\n3#\nget_ipython().run_line_magic(%r, %r)\n4#\n5#" % (u'ls', u'foo#')),
+            ("1\n2\n3\n%ls foo\n4\n5", "1#\n2#\n3#\nget_ipython().magic(%r)\n4#\n5#" % u'ls foo#'),
         ]:
             out = isp.transform_cell(raw)
             self.assertEqual(out.rstrip(), expected.rstrip())
@@ -495,7 +496,7 @@ class IPythonInputTestCase(InputSplitterTestCase):
 if __name__ == '__main__':
     # A simple demo for interactive experimentation.  This code will not get
     # picked up by any test suite.
-    from IPython.core.inputsplitter import IPythonInputSplitter
+    from IPython.core.inputsplitter import InputSplitter, IPythonInputSplitter
 
     # configure here the syntax to use, prompt and whether to autoindent
     #isp, start_prompt = InputSplitter(), '>>> '
@@ -508,7 +509,7 @@ if __name__ == '__main__':
         while True:
             prompt = start_prompt
             while isp.push_accepts_more():
-                indent = ' '*isp.get_indent_spaces()
+                indent = ' '*isp.indent_spaces
                 if autoindent:
                     line = indent + input(prompt+indent)
                 else:
@@ -529,38 +530,38 @@ if __name__ == '__main__':
 # Tests for cell magics support
 
 def test_last_blank():
-    assert isp.last_blank("") is False
-    assert isp.last_blank("abc") is False
-    assert isp.last_blank("abc\n") is False
-    assert isp.last_blank("abc\na") is False
+    nt.assert_false(isp.last_blank(''))
+    nt.assert_false(isp.last_blank('abc'))
+    nt.assert_false(isp.last_blank('abc\n'))
+    nt.assert_false(isp.last_blank('abc\na'))
 
-    assert isp.last_blank("\n") is True
-    assert isp.last_blank("\n ") is True
-    assert isp.last_blank("abc\n ") is True
-    assert isp.last_blank("abc\n\n") is True
-    assert isp.last_blank("abc\nd\n\n") is True
-    assert isp.last_blank("abc\nd\ne\n\n") is True
-    assert isp.last_blank("abc \n \n \n\n") is True
+    nt.assert_true(isp.last_blank('\n'))
+    nt.assert_true(isp.last_blank('\n '))
+    nt.assert_true(isp.last_blank('abc\n '))
+    nt.assert_true(isp.last_blank('abc\n\n'))
+    nt.assert_true(isp.last_blank('abc\nd\n\n'))
+    nt.assert_true(isp.last_blank('abc\nd\ne\n\n'))
+    nt.assert_true(isp.last_blank('abc \n \n \n\n'))
 
 
 def test_last_two_blanks():
-    assert isp.last_two_blanks("") is False
-    assert isp.last_two_blanks("abc") is False
-    assert isp.last_two_blanks("abc\n") is False
-    assert isp.last_two_blanks("abc\n\na") is False
-    assert isp.last_two_blanks("abc\n \n") is False
-    assert isp.last_two_blanks("abc\n\n") is False
+    nt.assert_false(isp.last_two_blanks(''))
+    nt.assert_false(isp.last_two_blanks('abc'))
+    nt.assert_false(isp.last_two_blanks('abc\n'))
+    nt.assert_false(isp.last_two_blanks('abc\n\na'))
+    nt.assert_false(isp.last_two_blanks('abc\n \n'))
+    nt.assert_false(isp.last_two_blanks('abc\n\n'))
 
-    assert isp.last_two_blanks("\n\n") is True
-    assert isp.last_two_blanks("\n\n ") is True
-    assert isp.last_two_blanks("\n \n") is True
-    assert isp.last_two_blanks("abc\n\n ") is True
-    assert isp.last_two_blanks("abc\n\n\n") is True
-    assert isp.last_two_blanks("abc\n\n \n") is True
-    assert isp.last_two_blanks("abc\n\n \n ") is True
-    assert isp.last_two_blanks("abc\n\n \n \n") is True
-    assert isp.last_two_blanks("abc\nd\n\n\n") is True
-    assert isp.last_two_blanks("abc\nd\ne\nf\n\n\n") is True
+    nt.assert_true(isp.last_two_blanks('\n\n'))
+    nt.assert_true(isp.last_two_blanks('\n\n '))
+    nt.assert_true(isp.last_two_blanks('\n \n'))
+    nt.assert_true(isp.last_two_blanks('abc\n\n '))
+    nt.assert_true(isp.last_two_blanks('abc\n\n\n'))
+    nt.assert_true(isp.last_two_blanks('abc\n\n \n'))
+    nt.assert_true(isp.last_two_blanks('abc\n\n \n '))
+    nt.assert_true(isp.last_two_blanks('abc\n\n \n \n'))
+    nt.assert_true(isp.last_two_blanks('abc\nd\n\n\n'))
+    nt.assert_true(isp.last_two_blanks('abc\nd\ne\nf\n\n\n'))
 
 
 class CellMagicsCommon(object):
@@ -568,12 +569,12 @@ class CellMagicsCommon(object):
     def test_whole_cell(self):
         src = "%%cellm line\nbody\n"
         out = self.sp.transform_cell(src)
-        ref = "get_ipython().run_cell_magic('cellm', 'line', 'body')\n"
-        assert out == ref
-
+        ref = u"get_ipython().run_cell_magic({u}'cellm', {u}'line', {u}'body')\n"
+        nt.assert_equal(out, py3compat.u_format(ref))
+    
     def test_cellmagic_help(self):
         self.sp.push('%%cellm?')
-        assert self.sp.push_accepts_more() is False
+        nt.assert_false(self.sp.push_accepts_more())
 
     def tearDown(self):
         self.sp.reset()
@@ -584,59 +585,21 @@ class CellModeCellMagics(CellMagicsCommon, unittest.TestCase):
 
     def test_incremental(self):
         sp = self.sp
-        sp.push("%%cellm firstline\n")
-        assert sp.push_accepts_more() is True  # 1
-        sp.push("line2\n")
-        assert sp.push_accepts_more() is True  # 2
-        sp.push("\n")
+        sp.push('%%cellm firstline\n')
+        nt.assert_true(sp.push_accepts_more()) #1
+        sp.push('line2\n')
+        nt.assert_true(sp.push_accepts_more()) #2
+        sp.push('\n')
         # This should accept a blank line and carry on until the cell is reset
-        assert sp.push_accepts_more() is True  # 3
-
-    def test_no_strip_coding(self):
-        src = '\n'.join([
-            '%%writefile foo.py',
-            '# coding: utf-8',
-            'print(u"üñîçø∂é")',
-        ])
-        out = self.sp.transform_cell(src)
-        assert "# coding: utf-8" in out
-
+        nt.assert_true(sp.push_accepts_more()) #3
 
 class LineModeCellMagics(CellMagicsCommon, unittest.TestCase):
     sp = isp.IPythonInputSplitter(line_input_checker=True)
 
     def test_incremental(self):
         sp = self.sp
-        sp.push("%%cellm line2\n")
-        assert sp.push_accepts_more() is True  # 1
-        sp.push("\n")
+        sp.push('%%cellm line2\n')
+        nt.assert_true(sp.push_accepts_more()) #1
+        sp.push('\n')
         # In this case, a blank line should end the cell magic
-        assert sp.push_accepts_more() is False  # 2
-
-
-indentation_samples = [
-    ('a = 1', 0),
-    ('for a in b:', 4),
-    ('def f():', 4),
-    ('def f(): #comment', 4),
-    ('a = ":#not a comment"', 0),
-    ('def f():\n    a = 1', 4),
-    ('def f():\n    return 1', 0),
-    ('for a in b:\n'
-     '   if a < 0:'
-     '       continue', 3),
-    ('a = {', 4),
-    ('a = {\n'
-     '     1,', 5),
-    ('b = """123', 0),
-    ('', 0),
-    ('def f():\n    pass', 0),
-    ('class Bar:\n    def f():\n        pass', 4),
-    ('class Bar:\n    def f():\n        raise', 4),
-]
-
-def test_find_next_indent():
-    for code, exp in indentation_samples:
-        res = isp.find_next_indent(code)
-        msg = "{!r} != {!r} (expected)\n Code: {!r}".format(res, exp, code)
-        assert res == exp, msg
+        nt.assert_false(sp.push_accepts_more()) #2
