@@ -360,25 +360,12 @@ def test_classic_prompt():
     for example in syntax_ml['multiline_datastructure_prompt']:
         transform_checker(example, ipt.classic_prompt)
 
-    # Check that we don't transform the second line if the first is obviously
-    # IPython syntax
-    transform_checker([
-        (u'%foo', '%foo'),
-        (u'>>> bar', '>>> bar'),
-    ], ipt.classic_prompt)
-
 
 def test_ipy_prompt():
     tt.check_pairs(transform_and_reset(ipt.ipy_prompt), syntax['ipy_prompt'])
     for example in syntax_ml['ipy_prompt']:
         transform_checker(example, ipt.ipy_prompt)
 
-    # Check that we don't transform the second line if we're inside a cell magic
-    transform_checker([
-        (u'%%foo', '%%foo'),
-        (u'In [1]: bar', 'In [1]: bar'),
-    ], ipt.ipy_prompt)
-
 def test_coding_cookie():
     tt.check_pairs(transform_and_reset(ipt.strip_encoding_cookie), syntax['strip_encoding_cookie'])
     for example in syntax_ml['strip_encoding_cookie']:
@@ -415,11 +402,6 @@ def test_assemble_python_lines():
        (u"2,", None),
        (None, u"a = [1,\n2,"),
       ],
-      [(u"a = '''", None),  # Test line continuation within a multi-line string
-       (u"abc\\", None),
-       (u"def", None),
-       (u"'''", u"a = '''\nabc\\\ndef\n'''"),
-      ],
     ] + syntax_ml['multiline_datastructure']
     for example in tests:
         transform_checker(example, ipt.assemble_python_lines)
