@@ -21,6 +21,7 @@ separate implementation).
 An example notebook is provided in our documentation illustrating interactive
 use of the system.
 """
+from __future__ import print_function
 
 #*****************************************************************************
 #       Copyright (C) 2005-2006 Fernando Perez <fperez@colorado.edu>
@@ -35,7 +36,8 @@ import threading
 
 from IPython import get_ipython
 from IPython.core.ultratb import AutoFormattedTB
-from logging import error, debug
+from IPython.utils.warn import error
+from IPython.utils.py3compat import string_types
 
 
 class BackgroundJobManager(object):
@@ -86,7 +88,6 @@ class BackgroundJobManager(object):
         self._s_running   = BackgroundJobBase.stat_running_c
         self._s_completed = BackgroundJobBase.stat_completed_c
         self._s_dead      = BackgroundJobBase.stat_dead_c
-        self._current_job_id = 0
 
     @property
     def running(self):
@@ -116,7 +117,7 @@ class BackgroundJobManager(object):
         The given expression is passed to eval(), along with the optional
         global/local dicts provided.  If no dicts are given, they are
         extracted automatically from the caller's frame.
-
+        
         A Python statement is NOT a valid eval() expression.  Basically, you
         can only use as an eval() argument something which can go on the right
         of an '=' sign and be assigned to a variable.
@@ -135,7 +136,7 @@ class BackgroundJobManager(object):
 
           job_manager.new(myfunc, x, y, kw=dict(z=1))
 
-        The reason for this asymmetry is that the new() method needs to
+        The reason for this assymmetry is that the new() method needs to
         maintain access to its own keywords, and this prevents name collisions
         between arguments to new() and arguments to your own functions.
 
@@ -171,7 +172,7 @@ class BackgroundJobManager(object):
         if callable(func_or_exp):
             kw  = kwargs.get('kw',{})
             job = BackgroundJobFunc(func_or_exp,*args,**kw)
-        elif isinstance(func_or_exp, str):
+        elif isinstance(func_or_exp, string_types):
             if not args:
                 frame = sys._getframe(1)
                 glob, loc = frame.f_globals, frame.f_locals
@@ -188,11 +189,10 @@ class BackgroundJobManager(object):
 
         if kwargs.get('daemon', False):
             job.daemon = True
-        job.num = self._current_job_id
-        self._current_job_id += 1
+        job.num = len(self.all)+1 if self.all else 0
         self.running.append(job)
         self.all[job.num] = job
-        debug('Starting job # %s in a separate thread.' % job.num)
+        print('Starting job # %s in a separate thread.' % job.num)
         job.start()
         return job
 
